import logging
import os
from re import match


def rpm_qa():
    return os.popen('rpm -qa')


evr_pattern = r'(?P<name>.*?)-(?P<version>\d+\.\d+\.\d+)-(?P<release>[^\.]+)\.(?P<arch>.+)'
payara_version_pattern = r'Thank you for downloading Payara Server (.*).'


def get_rpm_versions(prefix):
    """Get the versions of the RPMs installed on the system."""
    rpm_versions = {}
    for line in rpm_qa():
        if line.startswith(prefix):
            logging.debug(f'Found RPM matching prefix {prefix}: {line}')
            try:
                evr = match(evr_pattern, line)
                version = evr.group('version')
                module = evr.group('name')
                release = evr.group('release')
                # Adding the release part of the EVR is not very informative for DANS modules,
                # because it is always 1 for releases, except for SNAPSHOTs, in which case it is
                # important to know the exact SNAPSHOT version.
                rpm_versions[module] = f'{version}-{release}' if release != '1' else f'{version}'
            except:
                rpm_versions[module] = 'ERROR'

    return rpm_versions


def get_dataverse_version(dataverse_application_path):
    with open(os.path.join(dataverse_application_path, 'WEB-INF', 'classes', 'META-INF',
                           'microprofile-config.properties'), 'r') as f:
        try:
            for line in f:
                if 'dataverse.version' in line:
                    return (line.split('=')[1]).strip()
        except:
            return 'ERROR'


def get_dataverse_build_number(dataverse_application_path):
    with open(os.path.join(dataverse_application_path, 'WEB-INF', 'classes', 'BuildNumber.properties'), 'r') as f:
        try:
            for line in f:
                if 'build.number' in line:
                    return (line.split('=')[1]).strip()
        except:
            return 'ERROR'


def get_payara_version(payara_application_path):
    try:
        with open(os.path.join(payara_application_path, 'README.txt'), 'r') as f:
            line = next((line for line in f if match(payara_version_pattern, line)), None)
            payara_version = match(payara_version_pattern, line).group(1)
        return payara_version
    except:
        return 'ERROR'



import argparse

import yaml
import sys
import rich
from rich.console import Console
from rich.table import Table

from datastation.common.config import init
from datastation.common.version_info import get_rpm_versions, get_dataverse_version, get_dataverse_build_number, \
    get_payara_version


def get_config_version_info(config):
    if 'version_info' in config:
        return config['version_info']
    else:
        default_version_info = {
            'dans_rpm_module_prefix': 'dans.knaw.nl-',
            'dataverse_application_path': '/var/lib/payara5/glassfish/domains/domain1/applications/dataverse/',
            'payara_install_path': '/usr/local/payara5'
        }
        rich.print('WARNING: No version_info section in config file. Using default values.')
        rich.print('To get rid of this warning, add a version_info section to your config file:')
        yaml.dump({'version_info': default_version_info}, sys.stdout)
        return default_version_info


def main():
    config = init()

    parser = argparse.ArgumentParser(
        description='Gets the version of all Data Station components in this installation.')
    parser.add_argument('--json', dest='json', action='store_true', help='Output as JSON')
    args = parser.parse_args()

    version_info = get_config_version_info(config)

    components = get_rpm_versions(version_info['dans_rpm_module_prefix'])
    dataverse_version = get_dataverse_version(version_info['dataverse_application_path'])
    dataverse_build_number = get_dataverse_build_number(version_info['dataverse_application_path'])
    components['dataverse'] = f'{dataverse_version} build {dataverse_build_number}'
    payara_version = get_payara_version(version_info['payara_install_path'])
    components['payara'] = payara_version

    if args.json:
        rich.print(components)
        return
    else:
        table = Table(title="Data Station Component Versions")
        table.add_column("Component")
        table.add_column("Version")
        # Get a sorted list of keys into the components dictionary
        keys = list(components.keys())
        keys.sort()
        for key in keys:
            table.add_row(key, components[key])
        console = Console()
        console.print(table)


if __name__ == '__main__':
    main()
